import json
import logging
from banal.dicts import clean_dict
from datetime import datetime, date

from memorious.core import connect_redis

QUEUE_EXPIRE = 84600 * 14
log = logging.getLogger(__name__)


class Base(object):
    conn = connect_redis()


def unpack_int(value):
    try:
        return int(value)
    except Exception:
        return 0


def pack_datetime(value):
        if value is not None:
            return str(value)


def pack_now():
    return pack_datetime(datetime.utcnow())


def unpack_datetime(value):
    if value is not None:
        return datetime.strptime(value, "%Y-%m-%d %H:%M:%S.%f")


def delete_prefix(conn, prefix):
    idx = 0
    keys = conn.scan_iter(prefix)
    batch = []
    for idx, key in enumerate(keys):
        batch.append(key)
        if len(batch) > 100:
            conn.delete(*batch)
            batch = []
    if len(batch):
        conn.delete(*batch)
    log.info("Delete prefix %s: %s", prefix, idx)


class JSONEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, (datetime, date)):
            return obj.isoformat()
        if isinstance(obj, bytes):
            return obj.decode('utf-8')
        if isinstance(obj, set):
            return [o for o in obj]
        return json.JSONEncoder.default(self, obj)


def dump_json(data):
    if data is None:
        return
    data = clean_dict(data)
    return JSONEncoder().encode(data)


def load_json(encoded):
    if encoded is None:
        return
    return json.loads(encoded)



import logging

from memorious.model.common import Base, dump_json, load_json
from memorious.model.common import delete_prefix
from memorious.util import make_key

log = logging.getLogger(__name__)


class Tag(Base):
    """A simple key/value table used to store interim results."""

    @classmethod
    def save(cls, crawler, key, value):
        data = dump_json(value)
        key = make_key(crawler, "tag", key)
        cls.conn.set(key, data, ex=crawler.expire)

    @classmethod
    def find(cls, crawler, key):
        value = cls.conn.get(make_key(crawler, "tag", key))
        if value is not None:
            return load_json(value)

    @classmethod
    def exists(cls, crawler, key):
        return cls.conn.exists(make_key(crawler, "tag", key))

    @classmethod
    def delete(cls, crawler):
        delete_prefix(cls.conn, make_key(crawler, "tag", "*"))



import pickle
import codecs
import logging
from hashlib import sha1

from memorious.model.common import Base, delete_prefix, QUEUE_EXPIRE
from memorious.util import make_key

log = logging.getLogger(__name__)


class SessionState(Base):
    """An HTTP session state."""

    @classmethod
    def save(cls, crawler, session):
        session = pickle.dumps(session)
        session = codecs.encode(session, 'base64')
        key = sha1(session).hexdigest()[:15]
        key = make_key(crawler, "session", key)
        cls.conn.set(key, session, ex=QUEUE_EXPIRE)
        return key

    @classmethod
    def get(cls, crawler, key):
        value = cls.conn.get(make_key(crawler, "session", key))
        if value is not None:
            session = codecs.decode(bytes(value, 'utf-8'), 'base64')
            return pickle.loads(session)

    @classmethod
    def delete(cls, crawler):
        delete_prefix(cls.conn, make_key(crawler, "session", "*"))
